@@ -1,13 +1,20 @@
 import os
 import logging
 from pymk.error import TaskAlreadyExists, NoDependencysInAClass, NotADependencyError
-from pymk.dependency import InnerFileExists, InnerFileChanged, AlwaysRebuild, Dependency
+from pymk.dependency import InnerFileExists, InnerFileChanged, AlwaysRebuild, Dependency, InnerLink
 
 logger = logging.getLogger('pymk')
 
 
 class TaskMeta(type):
     tasks = {}
+    args = {}
+    _instances = {}
+
+    def __call__(cls, *args, **kwargs):
+        if cls not in cls._instances:
+            cls._instances[cls] = super(TaskMeta, cls).__call__(*args, **kwargs)
+        return cls._instances[cls]
 
     def __init__(cls, name, bases, dct):
         def check_if_task_exists(name):
@@ -23,16 +30,18 @@ class TaskMeta(type):
         #-----------------------------------------------------------------------
         if name != 'Task':
             check_if_task_exists(name)
-            TaskMeta.tasks[name] = cls
+            TaskMeta.tasks[cls().getName()] = cls
             validate_dependency(cls)
 
     @classmethod
     def init(cls):
         """Init new task collection."""
         cls.tasks = {}
+        cls.args = {}
 
 
 class Task(object):
+
     """Base of all taks."""
 
     __metaclass__ = TaskMeta
@@ -43,26 +52,31 @@ class Task(object):
     hide = False
     hide_graph = False
 
-    _name = None
+    name = None
     detailed = []
-    _runned = {}
+    _runned = False
+    _args = {}
+    _error = False
 
     @classmethod
-    def name(cls):
-        """name() -> str
+    def getName(cls):
+        """getName() -> str
         Returns name of the tasks provided by class value _name, or just classname if _name == None.
         """
-        return cls.__name__
+        if cls.name:
+            return cls.name
+        else:
+            return cls.__name__
 
     @classmethod
     def _set_runned(cls, value):
-        cls._runned[cls.__name__] = value
-        return cls._runned[cls.__name__]
+        cls._runned = value
+        return cls._runned
 
     @classmethod
     def _get_runned(cls):
         try:
-            return cls._runned[cls.__name__]
+            return cls._runned
         except KeyError:
             return False
 
@@ -98,6 +112,12 @@ class Task(object):
         """run(log_uptodate = True): -> bool
         Test dependency of this task, and rebuild it if nessesery.
         """
+        def build_with_args_or_not(cls):
+            try:
+                cls().build(cls._args)
+            except TypeError:
+                cls().build()
+        #-----------------------------------------------------------------------
         cls.running_list_element = {
             'type': 'task',
             'data': cls,
@@ -110,15 +130,18 @@ class Task(object):
             from pymk.graph import running_list
             running_list.append(cls.running_list_element)
         if cls.test_dependencys(force and dependency_force) or force:
-            logger.info(" * Building '%s'" % (cls.name()))
+            logger.info(" * Building '%s'" % (cls.getName()))
             try:
-                cls().build()
+                build_with_args_or_not(cls)
+            except Exception as er:
+                cls._error = True
+                raise er
             finally:
                 runned = cls._set_runned(True)
             return runned
         else:
             if log_uptodate:
-                logger.info(" * '%s' is up to date" % (cls.name()))
+                logger.info(" * '%s' is up to date" % (cls.getName()))
             return cls._set_runned(False)
 
     @classmethod
@@ -129,12 +152,16 @@ class Task(object):
     def dependency_FileChanged(cls):
         return InnerFileChanged(cls)
 
+    @classmethod
+    def dependency_Link(cls):
+        return InnerLink(cls)
+
     # -- graph specyfic --
     @classmethod
     def write_graph_detailed(cls, datalog):
         if not cls.hide_graph and not cls.name in cls.detailed:
-            cls.detailed.append(cls.name())
-            datalog.write('"%s" [%s];\n' % (cls.name(), cls.get_graph_details()))
+            cls.detailed.append(cls.getName())
+            datalog.write('"%s" [%s];\n' % (cls.getName(), cls.get_graph_details()))
 
     @classmethod
     def get_graph_details(cls):
@@ -144,6 +171,8 @@ class Task(object):
             shape = 'circle'
             color = 'grey'
         if cls._get_runned():
+            color = 'green'
+        if cls._error:
             color = 'red'
 
         return 'shape=%s, regular=1,style=filled,fillcolor=%s' % (shape, color)
