@@ -4,12 +4,15 @@ from . import task
 from . import dependency
 from . import extra
 from . import error
+from . import script
 
 all_test_cases = [
     task.TaskTest,
     task.TaskDependencyFileExistsTest,
     task.TaskDependencyFileChangedTest,
     task.TaskForcing,
+    task.TaskDependencyLinkTest,
+    task.TaskNameTest,
 
     dependency.FileDoesNotExistsDependencyTest,
     dependency.FileChangedDependencyTest,
@@ -22,6 +25,9 @@ all_test_cases = [
     extra.RunCmdTest,
 
     error.ErrorsTests,
+
+    script.TaskNameParseTest,
+    script.GraphTest,
 ]
 
 
